/* tspcie_dma.h
 * This is a library for DMA transfers to or from an FPGA that has the TS
 * PCIe core.  At this time the TS-4300 is the only product using this 
 * system.  To use DMA, you must first know which DMA channel is connected to
 * the hardware you are trying to talk to.  The TS PCIe core has up to 8
 * channels, each of which is hard-coded to a single hardware core.  The
 * default lineup at this time is:
 */
#define PCIE_CHAN_SD 0
#define PCIE_CHAN_FLASH 1
#define PCIE_CHAN_CAN 2
#define PCIE_CHAN_XUART 3 // XUART blockram region
#define PCIE_CHAN_MUXBUS_16 4 // 16 bit bus
#define PCIE_CHAN_MUXBUS_8 5 // 8 bit bus
/* Channels 6 and 7 are reserved.  Note that opencore designers can connect
 * the channels however they want, and if they change the connections, these
 * numberings need to be changed accordingly.  For example, a system with
 * several high bandwidth PC/104 peripherals might want an 8 bit MUXBUS 
 * channel for each of them.  This numbering just reflects the stock FPGA.
 *
 * To perform DMA transfers:
 * 1. Declare a buffer for DMA tranfers.
 *
 * 2. Declare a struct tspcie_dma.  This will be the argument to all API calls.
 *    Fill in all the fields except pa which is used internally.  See below.
 *
 * 3. Call tspcie_init4300.  You do not need to call this function agian
 *    unless you get a new *buf.
 *
 * 4. If you are doing a write, fill this buffer with write data.
 *
 * 5. Call tspcie_prep4300.  This must be done for each new transfer.
 *
 * 6. Call tspcie_start.
 *
 * The *buf is the buffer to or from which you will be doing DMA transfers,
 * and sz is the size of the transfers in bytes.  The adr field is the address
 * in FPGA space to or from which you are moving data.   For some cores, the
 * FPGA will ignore this since there is only one address you would stream from
 * and it is hard-coded, but it doesn't hurt to fill it in  anyway.  The flags
 * field takes any of the following flags:
 */ 
#define PCIE_WRITE 0x1
#define PCIE_SLOW 0x2
#define PCIE_INCR 0x4
#define PCIE_16BIT 0x8
#define PCIE_8BIT 0x10
/* The transfer will be a read unless the write flag is set.  If the slow flag
 * is set, that means the DMA channel you are using is connected to an IRQ
 * line and it will not perform bus cycles except when the IRQ is asserted.
 * If the slow flag is not set, DMA will move data as fast as possible.  If the
 * incr flag is set, the FPGA address will increment with each cycle, and if
 * not, it will camp on one address.  In the future, other flags may be added
 * to support fancier tranfers.
 *
 * The minimum granularity of DMA transfers is 4 bytes.  DMA channels can be
 * connected to 16 bit or 8 bit devices, but in that case two or four bus
 * transactions will be condensed into one 32 bit transaction.  So in all cases
 * the sz argument must be a multiple of 4 and the adr argument should
 * normally be 4 byte aligned.
 *
 * There is an exception to this when the incr flag is not set and the target
 * hardware is 8 or 16 bits wide.  In this case any address is clearly valid
 * for an 8 bit target and any even address is valid for a 16 bit target.  If
 * you are doing non-incrementing transactions from an 8 bit or 16 bit target,
 * you must set the 8bit or 16bit flag so that the DMA core handles this
 * correctly.  Even in these cases, sz must be a multiple of 4.  If incr is
 * set, these flags must not be set.
 *
 * The buffer must be 32 byte aligned and preferably 4096 byte aligned.  It 
 * cannot cross a 4096 byte page boundary.
 *
 * After starting a transfer, you can switch to other tasks while it is in
 * progress.  At any time after starting a transfer, you can call the
 * tspcie_status function.  This function returns 0 if the transfer is
 * complete.  If not, it returns a positive number which has some vague
 * relationship to the number of bytes remaining to be transfered.  It can
 * be used as a rough estimate.
 *
 * Note that to achieve maximum bandwidth using this API, a ping-pong buffer
 * or multiple buffer scheme is recommended.  While DMA is in progress on one
 * buffer, you can be processing and calling tspcie_prep4300 on the other.
 * However, a naive usage with one 4KB buffer has been shown to be at least
 * 85% efficient.
 *
 * This API is only needed for DMA.  Single reads and writes to the FPGA can
 * happen during DMA without a problem.  DMA usage is strongly recommended for
 * streaming reads.  For writes, it is not as important, but it is still 
 * preferred so as not to clog the bus and waste CPU time.
 */

struct tspcie_dma {
	char *buf;
	unsigned int sz;
	unsigned int channel;
	unsigned int adr;
	unsigned int flags;
	unsigned int pa;
};

int tspcie_init4300(struct tspcie_dma *dma);
int tspcie_prep4300(struct tspcie_dma *dma);
int tspcie_start(struct tspcie_dma *dma);
int tspcie_status(struct tspcie_dma *dma);

